%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%                                                                        

%%%----------------------------------------------------------------------
%%% File    : webappmon.erl
%%% Author  : Martin G. <marting@erix.ericsson.se>
%%% Purpose : Frontend to the webbased version of appmon.
%%% Created : 24 Apr 2001 by Martin G. <marting@erix.ericsson.se>
%%%----------------------------------------------------------------------

-module(webappmon).

% The functioons that the user can call to interact with the
% Genserver
-export([init/1,handle_call/3,handle_cast/2,handle_info/2]).
-export([terminate/2,code_change/3]).

-export([node_info/2,application_info/2,application_env/2]).
-export([proc_info/2,trace/2]).
-export([start/0,stop/0,start_link/0]).

% Export the function that returns the configdata to webtool
-export([configData/0]).
-behavoiur(genserver).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start the genserver                                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


start_link()->
    gen_server:start_link({local,webappmon_server},webappmon,[],[]).
start()->
    gen_server:start({local,webappmon_server},webappmon,[],[]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stop the genserver                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

stop()->
    gen_server:call(webappmon_server,stop,1000).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the page that shows the nodes and the apps on the sel node       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
node_info(Env,Input)->
    gen_server:call(webappmon_server,{node_data,Env,Input}).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the application process tree                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
application_info(Env,Input)->
    gen_server:call(webappmon_server,{app_data,Env,Input}).




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the page that shows the data about the process                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
proc_info(Env,Input)->
    gen_server:call(webappmon_server,{proc_data,Env,Input}).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the spec on the app                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
application_env(Env,Input)->
    gen_server:call(webappmon_server,{app_env,Env,Input}).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Toggle the trace flag for the selected process                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
trace(Env,Input)->
    gen_server:call(webappmon_server,{trace,Env,Input}).




configData()->
    {appmon,[{web_data,{"WebAppmon","/appmon/main_frame.html"}},
	     	{alias,{"/appmon",code:priv_dir(appmon)}},
		{alias,{erl_alias,"/erl",[webappmon]}},
		{start,{child,{backend,{process_info,start_link,[]},
			       permanent,100,worker,[process_info]}}},
	     {start,{child,{frontend,{webappmon,start_link,[]},
			    permanent,100,worker,[webappmon_server]}}}
	    ]}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Callback functions for the genserver                                 %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


init(Arg)->
    {ok,[]}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Create the different pages                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handle_call({node_data,Env,Input},From,State)->
    {reply,app_and_node_sel_page(Env,Input),State};

handle_call({app_data,Env,Input},From,State)->
    {reply,process_tree_page(Env,Input),State};


handle_call({proc_data,Env,Input},From,State)->
    {reply,process_specifickation_page(Env,Input),State};

handle_call({app_env,Env,Input},From,State)->
    {reply,application_specifickation_page(Env,Input),State};


handle_call({trace,Env,Input},From,State)->
    {reply,toggle_trace(Env,Input),State}.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Shutdown the genserver                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
terminate(_,State)->
    ok.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Callback function currently not used ...                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handle_cast(_,State)->
    {noreply,State}.

handle_info(_,State)->
    {noreply,State}.

code_change(Old_vsn,State,Extra)->
    {ok,State}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%%Private functions to create the part of the sides that is common    %%
%%to all sides                                                        %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the Header for the page If we now the mimetype use that type %%
%%otherwise use text                                                  %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
header() ->
    header("text/html").
header(MimeType) ->
    "Content-type: " ++ MimeType ++ "\r\n\r\n".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
%%Create the Htmlheader sett the title of the side to nothing if      %%       
%%we dont know the name of the side                                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
html_header()->
   html_header("").

html_header(Part) ->    
    "<HTML>\n" ++
	"<HEAD>\n" ++
	"<TITLE>Appmon " ++ Part ++  "</TITLE>\n" ++
	"</HEAD>\n".


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Close the Html tag and if neccessay add some clean upp              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
html_end()->
    "</HTML>".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%%The functions that creates the whole pages by collecting the        %%
%%neccessary data                                                     %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Returns the page where the user sees which nodes and apps that      %%
%% are availible for monitoring                                       %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
app_and_node_sel_page(Env,Input)->
    [header(),html_header(),node_body(httpd:parse_query(Input)),html_end()].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Returns the process tree for the application whose name is the first%%
%%value in the Inputs key/value list                                  %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
process_tree_page(Env,Input)->
    Browser=getBrowser(Env),
    [header(),html_header(),
     application_javascript(httpd:parse_query(Input),Browser),
     application_body(httpd:parse_query(Input),Browser),
     html_end()].
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Send trace on or off to the process thats pid is the third arg of   %%
%%the  inputs key/val list. Then it returns the process tree for the  %%
%% application that is the first key/val pair of input                %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
toggle_trace(Env,Input)->
    Browser=getBrowser(Env),
    send_trace(httpd:parse_query(Input)),
   [header(),html_header(),
     application_javascript(httpd:parse_query(Input),Browser),
     application_body(httpd:parse_query(Input),Browser),html_end()].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Creates the page that shows all information about the process that  %%
%%that is the first arg of th input key/val pairs                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
process_specifickation_page(Env,Input)->
    [header(),html_header(),
     process_body(httpd:parse_query(Input)),html_end()].

application_specifickation_page(Env,Input)->
    [header(),html_header(),application_env_body(httpd:parse_query(Input)),html_end()].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%% The Private functions that do the job                              %%
%% To build the the page that shows the applications                  %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Build the body of the side that shows the node name and the         %%
%%application list                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

node_body([{"node",Node}|_Rest])->
    case  process_info:is_node(Node) of
	{true,Controlled_node,Name} ->
	    "<BODY>" ++ node_selections_javascripts() ++ (node_selection(Controlled_node)) ++ (node_title(Name)) ++ 
		(application_tree(Controlled_node,Name)) ++  "<BODY>";
	       
	{false,Server_node,Name} ->
	    "<BODY>" ++ node_selections_javascripts() ++ (node_selection(Server_node)) ++ (node_title(Name)) ++ 
		(application_tree(Server_node,Name)) ++ "<BODY>"
    end;


%%case we didnt get a node use the current

node_body(_Whatever)->
    node_body([{atom_to_list(node),atom_to_list(node())}]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Returns the javascript that sets a new node to monitor              %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

node_selections_javascripts()->
    "<SCRIPT>
      function node_selected()
      {                                     
         parent.frames.main.location=\"../../appmon/start_info.html\"
         window.location =\"./node_info?node=\" + " ++ 
           "document.node_selection.nodes[document.node_selection.nodes.selectedIndex].value;
      }
      </SCRIPT>".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Insert the html kode that shows the combobox where the user can     %%
%%select another node to monitor                                      %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

node_selection(Node)->
    " <FORM ACTION=\"./node_info\" NAME=node_selection>\n
           <TABLE WIDTH=\"100%\" BORDER=\"0\">\n
              <TR><TD ALIGN=\"center\">\n 
                  <SELECT NAME=nodes onChange=\"node_selected()\">\n" ++
	                  print_nodes(order_nodes(Node,process_info:get_nodes())) ++
                    "</SELECT>\n
              </TD></TR>\n
            </TABLE>\n
          </FORM>".
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Add the node we are working with in the beginning of the list and   %%
%%remove it from other places so its always the first in the listbox  %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
order_nodes(Node,Node_list)->
    [Node|lists:delete(Node,Node_list)].
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Take the list of nodes and make it to a list of options to the      %%
%%combobox                                                            %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
print_nodes([])->
    [];
print_nodes([Node|Rest])->
    "<OPTION value=\"" ++ atom_to_list(Node) ++ "\">" ++ atom_to_list(Node) ++ "\n" ++
    print_nodes(Rest).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the header of the node info page i.e. the name of the node   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

node_title(Node)->
   " <TABLE WIDTH=\"100%\" BORDER=\"0\">
	<TR><TD ALIGN=\"center\"><FONT SIZE=5>Applications</FONT></TD></TR>
    </TABLE>\n".


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Build the body of the side that shows the node i.e the application  %%
%%list                                                                %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Node and Node_name are the same just different types Node are the atom  name
%%the list of the node name  

application_tree(Node,Node_name)->
    Fun=fun(Name)->
	    Name_str =atom_to_list(Name),
	    "<LI><A HREF=\"./application_info?name=" ++ Name_str  ++ 
		    "&mode=all&node=" ++ Node_name ++ "\" TARGET=main><B>" 
		    ++ Name_str ++  "</B></A>" ++ print_space(15-length(Name_str),[]) ++
		    "<A HREF=\"./application_env?name=" ++ Name_str ++ "&node=" 
		    ++ Node_name ++ "\" TARGET=\"main\"><FONT SIZE=2>spec</FONT></A></LI>\n" 
	end,
    "<UL>" ++ 
	lists:map(Fun, (process_info:get_applications(Node))) ++
	"</UL>" ++
	"<FORM Name=reload>" ++
	"<INPUT TYPE=\"button\" onClick=\"node_selected()\" 
              VALUE=\"Reload\">\n" ++
	"</FORM>" ++
	"<!--<A HREF=\"../../appmon/application_help.html\" TARGET=main>Help</A>-->".


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Print spaces between the application name and the spec link        %%
%%                                                                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
print_space(N,Space)when N >0 ->
    print_space(N-1,"&nbsp;" ++ Space);
print_space(N,Space)->
    Space.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%% The Private functions that do the job                              %%
%% To build the the page that shows process in an application         %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Generates the javascript that govern the look of the page that      %%
%%the processes of an application                                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%Args is the list whit input args should be App Mode, Node
application_javascript(Args,Browser)when length(Args)>=3 ->
    Vars=
	"<SCRIPT>
            var proc;
            var app=\"" ++ element(2,lists:nth(1,Args)) ++ "\";
            var node=\"" ++ element(2,lists:nth(3,Args)) ++ "\";",
    CommonFuncs=
	"
         function show_process_info() 
         {
            if(proc.indexOf(\"#Port\")== -1)
            {
    	      if(proc.charAt(0)==\'<\')
                 window.location=\"./proc_info?name=\" + proc + \"&node=\" + node  
              else
                {
                  start=proc.indexOf(\"<\");
                  endpoint=proc.lastIndexOf(\">\");
                  window.location=\"./proc_info?name=\" + proc.slice(start,endpoint+1) + \"&node=\" + node  ;
               }
           }          
        }

        function trace()
        {
           if(proc.charAt(0)==\'<\')
              window.location=\"./trace?name=\" + app +  \"&mode=\" + get_mode() +  \"&node=\" + node + \"&proc=\" +  proc;
           else
           {
              start=proc.indexOf(\"<\");
              endpoint=proc.lastIndexOf(\">\");
              window.location=\"./trace?name=\" + app + \"&mode=\" + get_mode() + \"&node=\" + node + \"&proc=\" + 
                 proc.slice(start,endpoint+1) ;
           }         
        
       }
   
       function reload_page()\n
       { 
          window.location=\"./application_info?name=\" + app + \"&mode=\" + get_mode() + \"&node=\" + node ; 
       }
       function get_mode()
       {
          for (i= 0; i < document.reload_form.mode.length; i++)
          {
             if (document.reload_form.mode[i].checked)
                  return(document.reload_form.mode[i].value);
          }
          return(\"all\");
       }",
    BrowserSpecificScripts=case Browser of 
			       nav->
				   navJavaScript();
			       _Other ->
				   ieJavaScript()
			   end,
    
    Vars++CommonFuncs++BrowserSpecificScripts++"</SCRIPT>";
application_javascript(_,_)->
    "<SCRIPT></SCRIPT>".

navJavaScript()->
    "function showMenu(item)\n
     {
       var tempProc;
       tempProc=item.name;
       if( tempProc.indexOf(\"Port :\") == -1)
       {
         proc=item.name; \n
         if( proc.indexOf(\"untracable\") != 0)
	 {
	    window.document.menu_layer.left=item.left;\n
	    window.document.menu_layer.top=item.top+12;\n
	    window.document.menu_layer.visibility=\"visible\";\n
	    window.document.menu_layer2.visibility=\"hidden\";\n                     
	 }
	 else
	 {
	    window.document.menu_layer2.left=item.left;\n
	    window.document.menu_layer2.top=item.top+12;\n
	    window.document.menu_layer2.visibility=\"visible\";\n
	    window.document.menu_layer.visibility=\"hidden\";\n                     
         }
      }
   }
   function hideMenu()
   {
      window.document.menu_layer.visibility=\"hidden\";
   }
   function hideMenu2()\n
   {
  window.document.menu_layer2.visibility=\"hidden\";\n                     
  }\n".

ieJavaScript()->
    "function showMenu(item)
     {
       var tempProc;
       tempProc=item.id;
       if( tempProc.indexOf(\"Port :\") != 0)
       {
         proc=item.id; \n
         if( proc.indexOf(\"untracable\") != 0)
	 {
	    menu_layer.style.left=item.offsetLeft;
	    menu_layer.style.top=item.offsetTop+12;
	    menu_layer.style.visibility=\"visible\";
	    menu_layer2.style.visibility=\"hidden\";                     
	 }
	 else
	 {
	    menu_layer2.style.left=item.offsetLeft;
	    menu_layer2.style.top=item.offsetTop+12;

	    menu_layer2.style.visibility=\"visible\";
	    menu_layer.style.visibility=\"hidden\";                     
         }
      }
   }
   function hideMenu()
   {
     X=event.x;
     Y=event.y;
     MinX=menu_layer.style.left;
     MaxX=menu_layer.style.left+menu_layer.style.width;
     MinY=menu_layer.style.top;
     MaxY=menu_layer.style.top+menu_layer.style.height;
     if(Y<=MinY||Y>=MaxY)
          menu_layer.style.visibility=\"hidden\";
     if(X<=MinX||X>=MaxX)
         menu_layer.style.visibility=\"hidden\";
     //alert(\"X: \"+X+\" Y:\"+Y + \"MinX:\"+MinX+\" MaxX:\"+MaxX+\" MinY:\"+MinY +\" MaxY:\"+MaxY)
   }
   function hideMenu2()\n
   {
     X=event.x; 
     Y=event.y;
     MinX=menu_layer2.style.left;
     MaxX=menu_layer2.style.left+menu_layer2.style.width;
     MinY=menu_layer2.style.top;
     MaxY=menu_layer2.style.top+menu_layer2.style.height;
     if(Y<=MinY||Y>=MaxY)
          menu_layer2.style.visibility=\"hidden\";
     if(X<=MinX||X>=MaxX)
         menu_layer2.style.visibility=\"hidden\";
   }\n".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the body i.e the process tree for the applications whose name%%
%% is the second arg in the first tupole                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%args is the list [{_,Appname},{_,Mode},{_Node}]
application_body(Args,Browser)when list(Args),length(Args) >= 3 ->
    App=element(2,lists:nth(1,Args)),
    Mode=element(2,lists:nth(2,Args)),
    Node=element(2,lists:nth(3,Args)),
    "<BODY>" ++ mode_selection(Mode) ++ 
	selected_app_header(App,Node) ++ process_tree(App,Mode,Node,Browser) ++
	menu_layer(Browser) ++ 
	"</BODY>";
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%If the pattern above aint match then something is wrong             %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
application_body(_Whatever,Browser)->
    "Please use the links to the left".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the part of the process tree page side where the user can    %%
%%select the mode the view the tree in.                               %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mode_selection(Mode)->
    "<FORM NAME=\"reload_form\">\n" ++
	"<TABLE>" ++
	"<TR>\n" ++
	"<!--<TD><INPUT TYPE=\"button\" NAME=\"refresh_but\" VALUE=\"Reload\" onClick=\"reload_page()\">
         &nbsp;&nbsp;&nbsp;&nbsp;</TD>\n-->" ++
	print_radio_buttons([{"all","All processes"},{"sup_child","Supervised processes"},
	{"sup","Only supervisors"}],Mode) ++
	"</TR>\n </TABLE>\n" ++
	"</FORM>".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Print the radiobuttons. if the mode is the one the current radio-but%%
%% represent set the one checked                                      %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

print_radio_buttons([],_)->
    [];
print_radio_buttons([{Mode,Name}|Rest],Mode)->
    "<TD><INPUT TYPE=\"radio\" NAME=\"mode\" CHECKED=\"true\" VALUE=\""++ 
	Mode ++"\" onClick=\"reload_page()\">&nbsp;&nbsp;" ++Name ++  "</TD>\n" ++
	print_radio_buttons(Rest,Mode);
print_radio_buttons([{Mode1,Name}|Rest],Mode)->
    "<TD><INPUT TYPE=\"radio\" NAME=\"mode\" VALUE=\""++ Mode1 ++
	"\" onClick=\"reload_page()\">&nbsp;&nbsp;" ++Name ++  "</TD>\n" ++
	print_radio_buttons(Rest,Mode).

       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%The part that shows the name of the application  that the process   %%
%% tree represent                                                     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
       
selected_app_header(App,Node)->
    {Year,Mon,Day}=date(),
    "<TABLE>
        <TR> 
          <TD>Node:</TD> 
          <TD>" ++Node ++"</TD>
        </TR>
        <TR> 
          <TD>Application:</TD> 
          <TD>" ++App ++"</TD>
        </TR>
       <TR>  
          <TD>Date:</TD> 
          <TD>" ++ integer_to_list(Day) ++ "/" ++ 
                 integer_to_list(Mon) ++"&nbsp;-&nbsp;"++ 
                 integer_to_list(Year)  ++ 
         "</TD> 
        </TR>
     </TABLE>
     <TABLE WIDTH=100%>
       <TR>
         <TD>
             <HR WIDTH=\"80%\">
             <!--<FONT SIZE=4>Process tree</FONT>
             <HR ALIGN=\"center\" WIDTH=\"80%\">-->
         </TD>
       </TR>
     </TABLE>".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%get the process tree from process_ifo and build the nested unordered%%
%%list that represent the applications process tree                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

  process_tree(App,Mode,Node,Browser)->
    case process_info:get_processes(list_to_atom(App),list_to_atom(Mode),list_to_atom(Node)) of
	unknown->
	    "Unknown application please update application tree";
	{Tree,Traced_dict} ->
	    "<UL>" ++ htmlify_tree(Tree,Traced_dict,1,Browser) ++ "</UL>"
    end.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Build each node in the tree and then build its children             %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
htmlify_tree({Pid,Childs,Childs2},Proc_tab,N,nav)->
    case ets:lookup(Proc_tab,Pid) of
	[] when N<3->
	    "<LI><LAYER NAME=\"untracable" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" >" 
		 ++ Pid ++ "</LAYER></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,nav) ++
		htmlify_sec_child(Childs2);
	[_Traced]->
	    "<LI><LAYER NAME=\"" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" ><FONT COLOR=\"#5555FF\">" ++ 
		Pid ++ "</FONT></LAYER></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,nav) ++
		htmlify_sec_child(Childs2);
	[]-> 
	    "<LI><LAYER NAME=\"" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" >"++ 
		Pid ++ "</LAYER></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,nav) ++
		htmlify_sec_child(Childs2)	      	 
    end;

htmlify_tree({Pid,Childs,Childs2},Proc_tab,N,OtherBrowser)->
    case ets:lookup(Proc_tab,Pid) of
	[] when N<3->
	    "<LI><DIV ID=\"untracable" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" >"
		++ Pid ++ "</DIV></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,OtherBrowser) ++
		htmlify_sec_child(Childs2);
	[_]->
	    "<LI><DIV ID=\"" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" ><FONT COLOR=\"#5555FF\">" ++ 
		Pid ++ "</FONT></DIV></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,OtherBrowser) ++
		htmlify_sec_child(Childs2);
	[]-> 
	    "<LI><DIV ID=\"" ++ Pid ++ "\"  onMouseOver=\"showMenu(this)\" >"++ 
		Pid ++ "</DIV></LI>" ++
		htmlify_prim_child(Childs,Proc_tab,N+1,OtherBrowser) ++
		htmlify_sec_child(Childs2)
    end.
    	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Make html of the list of primary childs                             %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
htmlify_prim_child([],Proc_tab,N,Browser)->
    [];

htmlify_prim_child(Childs,Proc_tab,N,Browser)->
    Fun=fun(Child)->
		htmlify_tree(Child,Proc_tab,N,Browser)
	end,
    "<UL>\n" ++ lists:map(Fun,Childs) ++ "</UL>\n".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Make html of hte list whit sedondary childs, they has no childs     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
htmlify_sec_child([])->
    [];

htmlify_sec_child(Sec_child_list)->
    Htmlify_child=fun(Pid1)->
			  "<LI><FONT COLOR=\"#FF2222\">" ++ Pid1 ++ "</FONT></LI>\n"
		  end,
    "<UL>" ++ lists:map(Htmlify_child,Sec_child_list) ++ "</UL>\n".


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Make the popupmenu that the user can select what he will do on a    %%
%%process.                                                            %%
%%                                                                    %%
%%Note problably the only part that wont work in IE                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

menu_layer(nav)->
    "<LAYER NAME=\"menu_layer\" ID=\"men_lay\" LEFT=500 
            TOP=500 VISIBILITY=\"hidden\" BGCOLOR=\"#8899AA\" HEIGHT=75 \n" ++
	"WIDTH=150 onMouseOut=\"hideMenu()\" >\n" ++
	"<TABLE WIDTH=\"100%\" HEIGHT=\"100%\">\n" ++ 
	"<TR><TD BGCOLOR=\"#AABBCC\">\n" ++
	"<ILAYER><B>Select Action</B></ILAYER> \n" ++
	"</TD></TR>\n" ++
	"<TR><TD BGCOLOR=\"#AABBCC\" NAME=\"row1\" >\n" ++ 
	"<A HREF=\"./application_info\"  onMousedown=\"show_process_info()\">Show process info</A> \n" ++
	"</TD></TR>\n" ++
	"<TR><TD BGCOLOR=\"#AABBCC\"  NAME=\"row2\"> \n" ++
	"<A NAME=\"sendA\"  HREF=\"./application_info\" onMouseDown=\"trace()\" >Trace</A> \n" ++
	"</TD></TR> \n </TABLE> \n</LAYER>\n" ++

	"<LAYER NAME=\"menu_layer2\" ID=\"men_lay2\" LEFT=500 TOP=500 VISIBILITY=\"hidden\" 
             BGCOLOR=\"#8899AA\" HEIGHT=55 \n" ++
	"WIDTH=150 onMouseOut=\"hideMenu2()\" >\n" ++
	"<TABLE WIDTH=\"100%\" HEIGHT=\"100%\">\n" ++ 
	"<TR><TD BGCOLOR=\"#AABBCC\">\n" ++
	"<ILAYER><B>Select Action</B></ILAYER> \n" ++
	"</TD></TR>\n" ++
	"<TR><TD BGCOLOR=\"#AABBCC\" NAME=\"row1\" >\n" ++ 
	"<A HREF=\"./application_info\"  onMousedown=\"show_process_info()\">Show process info</A> \n" ++
	"</TD></TR>\n" ++
	"</TABLE> \n</LAYER>\n";

menu_layer(Browser)->
    LinkStyle="\"color:#0000EE; cursor:hand; text-decoration:underline\"",
    Style= "\"visibility:hidden; left:500; top:500; background-color:#8899AA; height:120; width:150; position:absolute\"",
    Style1="\"visibility:hidden; left:500; top:500; background-color:#8899AA; height:55; width:150; position:absolute\"",
    "<DIV ID=\"menu_layer\" Name=\"men_lay\" STYLE="++ Style ++"onMouseOut=\"hideMenu()\" >
	<TABLE WIDTH=\"100% \" HEIGHT=\"100%\" id=rowcol>
	<TR id=row><TD ID=col BGCOLOR=\"#AABBCC\">
	<U>Select Action</U> 
	</TD></TR>
	<TR id=row1><TD BGCOLOR=\"#AABBCC\" NAME=\"col1\" onMouseDown=\"show_process_info()\" STYLE="++ LinkStyle  ++">
          Show process info 
	</TD ></TR>
	<TR id=row2><TD BGCOLOR=\"#AABBCC\"  id=\"row2\" onMouseDown=\"trace()\" STYLE="++ LinkStyle  ++">
             Trace 
	</TD></TR>
        </TABLE> 
        </DIV>
	<DIV ID=\"menu_layer2\" NAME=\"men_lay2\" STYLE=" ++ Style1  ++ "onMouseOut=\"hideMenu2()\" >
	<TABLE WIDTH=\"100%\" HEIGHT=\"100%\" id=rowcol2> 
	<TR id=row21><TD BGCOLOR=\"#AABBCC\" id=col21>
	<B>Select Action</B> 
	</TD></TR>
	<TR id=row22><TD BGCOLOR=\"#AABBCC\" id=\"col221\"  onMousedown=\"show_process_info()\" STYLE="++ LinkStyle  ++">
              Show process info
        </TD></TR>
	</TABLE> \n</DIV>\n".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%% The Private functions that do the job                              %%
%% To build the the page that shows process data                      %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%The function that creates the collects the various part of the side %%
%%that shows information about a specific process, Pid_namer should be%%
%%the list representation of a pid                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
process_body(Args) when length(Args)==2->
    Pid=element(2,lists:nth(1,Args)),
    Node=element(2,lists:nth(2,Args)),
    "<BODY>"  ++ process_information_table(Pid,Node) ++ "</BODY>";

process_body(_)->
    "<BODY>Please dont call this side manually</BODY>".

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the table that shows the name of the pid to show extended    %%
%%info about                                                          %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Get the table that shows the extended info about a process          %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
process_information_table(Pid_name,Node)->
    case catch list_to_pid(Pid_name) of
	Pid when pid(Pid) -> 
	    get_process_table(Pid,Node);
	Other ->
	    "Not a process Pid"
    end.
	      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Create the table that shoows the extended info about processes      %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
get_process_table(Pid,Node_name) when list(Node_name)->
    Node=list_to_atom(Node_name),
    get_process_table(Pid,Node);

get_process_table(Pid,Node)when atom(Node)->
    case lists:member(Node,[node()|nodes()]) of
	true-> 
	    {Year,Mon,Day}=date(),
	    Proc_data=process_info:get_process_data(Pid,Node),
	    "<BR><BR><TABLE BORDER=1>
                <TR BGCOLOR=\"#8899AA\"><TD COLSPAN=3 ALIGN=\"center\" > 
                       <FONT size=6> Process" ++ pid_to_list(Pid)  ++  "</FONT>
               </TD></TR>
		<TR><TD NOWRAP=\"true\"><B>Date</B></TD>
                    <TD>" ++ integer_to_list(Day) ++ "/" ++ 
                             integer_to_list(Mon) ++"&nbsp;-&nbsp;"++ 
                             integer_to_list(Year) ++ "</TD>
                    <TD>&nbsp;</TD>
                 </TR>\n " ++
		process_proc_data(Proc_data) ++ "</TABLE><BR><BR>";
	_->
	    "Please try again the Node dont exists"
    end.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% The process information is quite messy tidi it up by creating a    %%
%%table that looks like                  key           val            %%
%%or if it has a list of key/val         Masterkey                    %%
%%                                                     Key        val %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
process_proc_data([])->
    [];
process_proc_data([{current_function,{Module,Function,Arg}}|Rest])->
    "<TR><TD NOWRAP=true><B>Current function:</B></TD><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>\n " ++
	"<TR><TD>&nbsp;</TD><TD>Module</TD><TD>" ++ atom_to_list(Module) ++ "</TD></TR>\n" ++
	"<TR><TD>&nbsp;</TD><TD>Function</TD><TD>" ++ atom_to_list(Function) ++ "</TD></TR>\n" ++
	"<TR><TD>&nbsp;</TD><TD>Arguments</TD><TD>" ++ integer_to_list(Arg) ++ "</TD></TR>\n" ++
	process_proc_data(Rest);

process_proc_data([{error_handler,Mod}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Error handler:</B></TD><TD>" ++ atom_to_list(Mod)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{group_leader,Grp}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Group leader:</B></TD><TD>" ++ pid_to_list(Grp)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{heap_size,Size}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Heap size:</B></TD><TD>" ++ integer_to_list(Size)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);


process_proc_data([{initial_call,{Module,Function,Arity}}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Initial call:</B></TD><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>\n " ++
	"<TR><TD>&nbsp;</TD><TD>Module</TD><TD>" ++ atom_to_list(Module) ++ "</TD></TR>\n" ++
	"<TR><TD>&nbsp;</TD><TD>Function</TD><TD>" ++ atom_to_list(Function) ++ "</TD></TR>\n" ++
	"<TR><TD>&nbsp;</TD><TD>Arguments</TD><TD>" ++ integer_to_list(Arity) ++ "</TD></TR>\n" ++
	process_proc_data(Rest);


process_proc_data([{links,List_of_Pids}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Links:</B></TD><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>\n " ++
	print_links(List_of_Pids) ++
	process_proc_data(Rest);

process_proc_data([{message_queue_len,Size}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Message queue length:</B></TD><TD>" ++ integer_to_list(Size)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);
       
process_proc_data([{priority,Level}|Rest])->
    "<TR><TD><B>Process priority:</B></TD><TD>" ++ atom_to_list(Level)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{reductions,Number}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Number of executed reductions:</B></TD><TD>" ++ integer_to_list(Number)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{registered_name,Name}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Process Name:</B></TD><TD>" ++ atom_to_list(Name)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{stack_size,Size}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Stack size:</B></TD><TD>" ++ integer_to_list(Size)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{status,Status}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Process status:</B></TD><TD>" ++ atom_to_list(Status)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{trap_exit,Boolean}|Rest])->
    "<TR><TD NOWRAP=\"true\" ><B>Trap Exit:</B></TD><TD>" ++ atom_to_list(Boolean)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{messages,Queue}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Message Queue:</B></TD><TD>" ++ io_lib:write(Queue)  ++ "</TD><TD>&nbsp;</TD></TR>\n " ++
	process_proc_data(Rest);

process_proc_data([{dictionary,Dict}|Rest])->
    "<TR><TD NOWRAP=\"true\"><B>Process dictionary:</B></TD><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>\n " ++
	get_dictionary_data(Dict) ++
	process_proc_data(Rest);

process_proc_data([H|Rest])->
    process_proc_data(Rest).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%In the process info there are the links to other processes print    %%
%% this pid                                                           %%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
print_links([])->
    [];
print_links([Pid|Rest])when pid(Pid) ->
    "<TR><TD>&nbsp;</TD><TD>To process:</TD><TD>" ++ pid_to_list(Pid) ++  "</TD></TR>\n " ++
	print_links(Rest);
print_links([Pid|Rest])when port(Pid) ->
    "<TR><TD>&nbsp;</TD><TD>To port:</TD><TD>" ++ erlang:port_to_list(Pid) ++  "</TD></TR>\n " ++
	print_links(Rest).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Fix the data in the process dictionary                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
get_dictionary_data([])->
    [];
get_dictionary_data([{Key,Val}|Dict])->
    "<TR><TD>&nbsp;</TD><TD>" ++ fix_type(Key) ++  "</TD><TD>" ++ fix_type(Val) ++ "</TD></TR>\n" ++
	get_dictionary_data(Dict).
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Interpret the type of the data and makr it to a list                %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

fix_type(Val)when atom(Val) ->
    atom_to_list(Val);
fix_type(Val)when float(Val) ->
    float_to_list(Val);
fix_type(Val)when integer(Val) ->
    integer_to_list(Val);
fix_type(Val)when list(Val) ->
    case io_lib:printable_list(Val) of
	true->
	    case Val of
		[]->
		    io_lib:write(Val);
		_->
		    Val
	    end;
	_->
	    io_lib:write(Val)
    end;
fix_type(Val)when pid(Val) ->
    pid_to_list(Val);
fix_type(Val)when port(Val) ->
    pid_to_list(Val);
fix_type(Val)when tuple(Val) ->
    io_lib:write(Val);
fix_type(Val) ->
    [].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%	
%% The Private functions that send the trace signal to the process    %%
%% thats the 4 member of the Arg list                                 %%
%%                                                                    %%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
send_trace(Args)when length(Args)>=4->
    {_,Proc}=lists:nth(4,Args),
    process_info:send_trace(Proc);


send_trace(_Args)->
    arg_error.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%	
%%Private functions that prints the application environment           %%	
%%                                                                    %%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

application_env_body(Args)when length(Args)>=2 ->
    App=element(2,lists:nth(1,Args)),
    Node=element(2,lists:nth(2,Args)),
    "<BODY>" ++ application_env_table(App,Node) ++ "</BODY>";


application_env_body(_)->
    "<BODY>Please dont call this side manually</BODY>".

application_env_table(App,Node)->
    case process_info:get_application_keys(list_to_atom(App),list_to_atom(Node)) of
	{ok,List}->
	   "<TABLE BORDER=1>" ++ application_env_head(App,Node) ++ print_key_list(List,[]) ++ "</TABLE>";
	_ ->
	    "Please try again,something went wrong"
    end.

application_env_head(App,Node)->
           "<TR BGCOLOR=\"#8899AA\"><TD ALIGN=\"center\" COLSPAN=3> 
                <FONT SIZE=6>" ++ App ++ "@" ++ Node ++ "</FONT>\n
           </TD></TR>
           <TR><TD COLSPAN=3>&nbsp</TD></TR>
          <TR BGCOLOR=\"#8899AA\">
             <TD><B>Key</B></TD><TD><B>Val/Sec. key</B></TD><TD><B>Sec. Val</B></TD>
         </TR>".
                  


print_key_list([],Result)->
    Result;

print_key_list([{application,Name}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Application name :",Name));

print_key_list([{description,Desc}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Description :",Desc));

print_key_list([{vsn,Ver}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Version :",Ver));

print_key_list([{id,Id}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("ID:",fix_type(Id)));

print_key_list([{modules,Mods}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Modules:","&nbsp;") ++ print_secondary_list(Mods,[]));

print_key_list([{maxP,Max}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Max nr of processes",fix_type(Max)));

print_key_list([{maxT,Max}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Max running sec:",fix_type(Max)));

print_key_list([{registered,Names}|Rest],Result)->    
    print_key_list(Rest,Result ++ print_key("Registered names:","&nbsp;") ++print_secondary_list(Names,[]));

print_key_list([{applications,Apps}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Depends on:","&nbsp") ++ print_secondary_list(Apps,[]));

print_key_list([{included_applications,Inc_apps}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Included applications:",fix_type(Inc_apps)));

print_key_list([{env,Env}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Environment:",fix_type(Env)));

print_key_list([{mod,Mod}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Application callback mod:",fix_type(Mod)));

print_key_list([{start_phases,Phase_arg}|Rest],Result)->
    print_key_list(Rest,Result ++ print_key("Application callback mod:",fix_type(Phase_arg)));

print_key_list([_|Rest],Result)->
    print_key_list(Rest,Result).


print_key(Label,Val)->
    "<TR>
        <TD><B>" ++ Label ++ "</B></TD><TD>" ++ Val ++  "</TD><TD>&nbsp;</TD>
    </TR>".


print_key2(Label,Val)->
    "<TR>
        <TD>&nbsp;</TD><TD>" ++ Label ++ "</TD><TD>" ++ Val ++  "</TD>
    </TR>".

print_secondary_list([],Result)->
    Result;
print_secondary_list([{Mod,Ver}|Rest],Result) ->
    print_secondary_list(Rest,Result ++ print_key2(fix_type(Mod),fix_type(Ver)));

print_secondary_list([Mod|Rest],Result) ->
    print_secondary_list(Rest,Result ++ print_key2(fix_type(Mod),"&nbsp;")).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                    %%
%%Returns either ie or nav depending on which browser is used         %%
%%                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


getBrowser(Env)->
        case lists:keysearch(http_user_agent,1,Env) of
	{value,{_,AgentString}}->
	    case mod_browser:getBrowser(AgentString) of
		{{msie,_},_}->
		    ie;
		{{mozilla,_},_}->
		    nav;
		{{netscape,_},_} ->
		    nav;
		_ ->
		    ie
	    end;
	_ ->
	    ie
    end.






























































